#!/usr/bin/perl
# 
# Copyright (C) 2015 Roland Laifer 
#                    Karlsruhe Institute of Technology (KIT), 
#                    Steinbuch Centre for Computing (SCC)
#                    76128 Karlsruhe, Germany
#                    Roland.Laifer (at) kit.edu
#
# ------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ------------------------------------------------------------------------
#
# $Log: files_2_ostids.pl,v $
# Revision 1.3  2015/12/23 14:37:29  rz54
# Added GPL
#
# Revision 1.2  2015/12/23 11:21:14  rz54
# Added option -O
#
# Revision 1.1  2015/11/17 16:14:00  rz54
# Initial revision
#
#
# $Id: files_2_ostids.pl,v 1.3 2015/12/23 14:37:29 rz54 Exp $

use strict;
use warnings;

use Getopt::Std;
our($opt_c, $opt_o, $opt_O, $opt_h, $opt_v);

# get script name
(my $myname = $0) =~ s#^.*/##;


sub usage {
    print "Usage: $myname -c <command> -o <ostnum> [-h] [-v]\n";
    print "$myname creates a list of object IDs of a dedicated OST from files\n";
    print <<'EOUSE1';
which are located by a provided command.
  -c <command>   execute command to locate files, e.g. 'find <dir> -user <name> -type f'
  -o <ostnum>    OST number for which object IDs should be displayed
  -O             print file names together with object ID
  -h             this help
  -v             verbose
EOUSE1
    exit 1;
}


getopts("c:o:Ohv") || usage;
usage if ( $opt_h );
usage unless ( $opt_c );
usage unless ( $opt_o || $opt_o == 0 );
usage unless ($#ARGV == -1);
unless ( $opt_o =~ /^\d+$/ ) {
    print STDERR "A number is required for option -o !\n";
    usage;
}

print "Executing: $opt_c\n" if ($opt_v);
open my $find_files_h, "-|", $opt_c or die "*** Command $opt_c failed: $!\n";
while ( my $file = <$find_files_h> ) {
    chomp $file;
    if ( -f $file ) {
	my $cmd = "/usr/bin/lfs getstripe -q -L \"$file\"";
	print "Executing: $cmd\n" if ($opt_v);
	open my $getstripe_h, "-|", $cmd or die "*** Command $cmd failed: $!\n";
	while ( my $line = <$getstripe_h> ) {
	    chomp $line;
	    if ( $line =~ /^\s*$opt_o\s+(\d+)\s+0x/ ) {
		my $objid = $1;
		print "Found matching line: $line\n" if ($opt_v);
		if ( $opt_O ) {
		    print "$1 $file\n";
		} else {
		    print "$1\n";
		}
	    }
	}
	close($getstripe_h);
    } else {
	print STDERR "Command $opt_c provided line ($file) which is not a file.\n";
	print STDERR "Exiting because of unexpected output line.\n";
	exit 1;
    }
}
close($find_files_h);

exit 0;
