#!/usr/bin/perl
# 
# Copyright (C) 2015 Roland Laifer 
#                    Karlsruhe Institute of Technology (KIT), 
#                    Steinbuch Centre for Computing (SCC)
#                    76128 Karlsruhe, Germany
#                    Roland.Laifer (at) kit.edu
#
# ------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ------------------------------------------------------------------------
#
# $Log: ost_obj_pattern_2_fid.pl,v $
# Revision 1.1  2015/10/29 14:59:35  rz54
# Initial revision
#
#
# $Id: ost_obj_pattern_2_fid.pl,v 1.1 2015/10/29 14:59:35 rz54 Exp $

use strict;
use warnings;

use Getopt::Std;
our($opt_p, $opt_h, $opt_v);

(my $myname = $0)	=~ s#^.*/##;

sub usage {
    print "$myname parses debugfs stat output (on STDIN), checks for each object\n";
    print "if a pattern exists and, if yes, prints FID (also object ID with option -v.\n";
    print "Data on STDIN is usually created with stat_ost_dev.pl\n";
    print "Usage: $myname [-h] -p <pattern> [-v]\n";
    print "       -p <pattern>    perl patten which has to match\n";
    print "       -h              this help\n";
    print "       -v              verbose\n";
    print "Example 1: $myname -p 'User:\\s*0\\s+'\n";
    print "Example 2: $myname -p 'Group:\\s*[0-9]{7,}\\s+'\n";
    exit 0;
}

# get options
getopts("p:hv") || usage;
usage if ( $opt_h );
usage unless ($#ARGV == -1);

unless ($opt_p) {
    print STDERR "Option -p is missing. Exiting...\n";
    exit 1;
}

# specify if patten matched
my $patternok = 0;
# object ID
my $objid; 

open my $stdin_h, "-" or die "*** Error opening STDIN failed: $!\n";
while (<$stdin_h>) {
    if (m|debugfs:\s*stat\s*/O/0/d\d+/(\d+)\s*$|) {
	$objid = $1;
	$patternok = 0;
#	print "Found object ID $objid\n";
    } elsif (m|$opt_p|) {
	$patternok = 1;
    } elsif (m|fid:.*parent=\[(\S+)\]|) {
	my $fileid = $1;
	if ($patternok) {
	    if ($opt_v) {
		print "Object ID: $objid, file ID: $fileid\n";
	    } else {
		print "$fileid\n";
	    }
	}
    }
}
